@@ -15,6 +15,7 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
+#include "glib-compat.h"
 #include "spice-client.h"
 #include "spice-common.h"
 
@@ -245,69 +246,49 @@ static void do_emit_main_context(GObject *object, int signum, gpointer params)
 
 /* ------------------------------------------------------------------ */
 
-static void mono_cursor(display_cursor *cursor, const guint8 *data)
+#ifdef DEBUG_CURSOR
+static void print_cursor(display_cursor *cursor, const guint8 *data)
 {
+    int x, y, bpl;
     const guint8 *xor, *and;
-    guint8 *dest;
-    int bpl, x, y, bit;
 
     bpl = (cursor->hdr.width + 7) / 8;
     and = data;
     xor = and + bpl * cursor->hdr.height;
-    dest  = (uint8_t *)cursor->data;
-    for (y = 0; y < cursor->hdr.height; y++) {
-        bit = 0x80;
-        for (x = 0; x < cursor->hdr.width; x++, dest += 4) {
-            if (and[x/8] & bit) {
-                if (xor[x/8] & bit) {
-                    /*
-                     * flip -> unsupported by x11, since XCreatePixmapCursor has
-                     * no invert functionality, only a mask, shape, background and
-                     * foreground colors. Use this checkerboard hack to get some
-                     * contrast for cursors in the guest that relied on invert for
-                     * the same contrast.
-                     */
-                    if ((x ^ y) & 1) {
-                        dest[0] = 0x30;
-                        dest[1] = 0x30;
-                        dest[2] = 0x30;
-                        dest[3] = 0xc0;
-                    } else {
-                        dest[0] = 0x50;
-                        dest[1] = 0x50;
-                        dest[2] = 0x50;
-                        dest[3] = 0x30;
-                    }
-                } else {
-                    /* unchanged -> transparent */
-                    dest[0] = 0x00;
-                    dest[1] = 0x00;
-                    dest[2] = 0x00;
-                    dest[3] = 0x00;
-                }
-            } else {
-                if (xor[x/8] & bit) {
-                    /* set -> white */
-                    dest[0] = 0xff;
-                    dest[1] = 0xff;
-                    dest[2] = 0xff;
-                    dest[3] = 0xff;
-                } else {
-                    /* clear -> black */
-                    dest[0] = 0x00;
-                    dest[1] = 0x00;
-                    dest[2] = 0x00;
-                    dest[3] = 0xff;
-                }
-            }
-            bit >>= 1;
-            if (bit == 0) {
-                bit = 0x80;
-            }
+
+    printf("data (%d x %d):\n", cursor->hdr.width, cursor->hdr.height);
+    for (y = 0 ; y < cursor->hdr.height; ++y) {
+        for (x = 0 ; x < cursor->hdr.width / 8; x++) {
+            printf("%02X", and[x]);
         }
         and += bpl;
-        xor += bpl;
+        printf("\n");
     }
+    printf("xor:\n");
+    for (y = 0 ; y < cursor->hdr.height; ++y) {
+        for (x = 0 ; x < cursor->hdr.width / 8; ++x) {
+            printf("%02X", xor[x]);
+        }
+        xor += bpl;
+        printf("\n");
+    }
+}
+#endif
+
+static void mono_cursor(display_cursor *cursor, const guint8 *data)
+{
+    int bpl = (cursor->hdr.width + 7) / 8;
+    const guint8 *xor, *and;
+    guint8 *dest;
+    dest = (uint8_t *)cursor->data;
+
+#ifdef DEBUG_CURSOR
+    print_cursor(cursor, data);
+#endif
+    and = data;
+    xor = and + bpl * cursor->hdr.height;
+    spice_mono_edge_highlight(cursor->hdr.width, cursor->hdr.height,
+                              and, xor, dest);
 }
 
 static guint8 get_pix_mask(const guint8 *data, gint offset, gint pix_index)
