@@ -546,12 +546,12 @@ static SpicePalette *palette_get(SpicePaletteCache *cache, uint64_t id)
         SPICE_CONTAINEROF(cache, SpiceDisplayChannelPrivate, palette_cache);
 
     /* here the returned pointer is weak, no ref given to caller.  it
-     * seems spice canvas usage is exclusively temporary, so it's ok
-     * (for now) */
+     * seems spice canvas usage is exclusively temporary, so it's ok.
+     * palette_release is a noop. */
     return cache_find(c->palettes, id);
 }
 
-static void palette_remove(SpicePaletteCache *cache, uint32_t id)
+static void palette_remove(SpicePaletteCache *cache, uint64_t id)
 {
     SpiceDisplayChannelPrivate *c =
         SPICE_CONTAINEROF(cache, SpiceDisplayChannelPrivate, palette_cache);
@@ -561,7 +561,7 @@ static void palette_remove(SpicePaletteCache *cache, uint32_t id)
 
 static void palette_release(SpicePaletteCache *cache, SpicePalette *palette)
 {
-    palette_remove(cache, palette->unique);
+    /* there is no refcount of palette, see palette_get() */
 }
 
 #ifdef SW_CANVAS_CACHE
@@ -722,9 +722,8 @@ static int create_canvas(SpiceChannel *channel, display_surface *surface)
         surface->shmid = -1;
     }
 
-    if (surface->shmid == -1) {
-        surface->data = spice_malloc(surface->size);
-    }
+    if (surface->shmid == -1)
+        surface->data = g_malloc0(surface->size);
 
     g_return_val_if_fail(c->glz_window, 0);
 
