@@ -65,11 +65,7 @@ struct _SpicePlaybackChannelClass {
     void (*playback_stop)(SpicePlaybackChannel *channel);
 
     /*< private >*/
-    /*
-     * If adding fields to this struct, remove corresponding
-     * amount of padding to avoid changing overall struct size
-     */
-    gchar _spice_reserved[SPICE_RESERVED_PADDING];
+    /* Do not add fields to this struct */
 };
 
 GType           spice_playback_channel_get_type(void);
