@@ -19,7 +19,8 @@
 # define SPICEGTK_DECODE_H_
 
 #include <glib.h>
-#include "canvas_base.h"
+
+#include "common/canvas_base.h"
 
 G_BEGIN_DECLS
 
