@@ -26,10 +26,13 @@
 
 G_BEGIN_DECLS
 
-struct wait_queue
+typedef struct _GCoroutine GCoroutine;
+
+struct _GCoroutine
 {
-    gboolean waiting;
-    struct coroutine *context;
+    struct coroutine coroutine;
+    guint wait_id;
+    guint condition_id;
 };
 
 /*
@@ -41,22 +44,18 @@ struct wait_queue
  * to busy wait on a timeout, since our condition is only checked
  * when some other source's state changes
  */
-typedef gboolean (*g_condition_wait_func)(gpointer);
-
-struct g_condition_wait_source
-{
-    GSource src;
-    struct coroutine *co;
-    g_condition_wait_func func;
-    gpointer data;
-};
+typedef gboolean (*GConditionWaitFunc)(gpointer);
 
 typedef void (*GSignalEmitMainFunc)(GObject *object, int signum, gpointer params);
 
-GIOCondition g_io_wait              (GSocket *sock, GIOCondition cond);
-gboolean     g_condition_wait       (g_condition_wait_func func, gpointer data);
-void         g_io_wakeup            (struct wait_queue *wait);
-GIOCondition g_io_wait_interruptible(struct wait_queue *wait, GSocket *sock, GIOCondition cond);
+GCoroutine*  g_coroutine_self           (void);
+void         g_coroutine_wakeup         (GCoroutine *coroutine);
+GIOCondition g_coroutine_socket_wait    (GCoroutine *coroutine,
+                                         GSocket *sock, GIOCondition cond);
+gboolean     g_coroutine_condition_wait (GCoroutine *coroutine,
+                                         GConditionWaitFunc func, gpointer data);
+void         g_coroutine_condition_cancel(GCoroutine *coroutine);
+
 void         g_signal_emit_main_context(GObject *object, GSignalEmitMainFunc func,
                                         int signum, gpointer params, const char *debug_info);
 void         g_object_notify_main_context(GObject *object, const gchar *property_name);
