@@ -113,7 +113,6 @@ struct _SpiceChannelPrivate {
     guint                       channel_watch;
     int                         tls;
 
-    int                         connection_id;
     int                         channel_id;
     int                         channel_type;
     SpiceLinkHeader             link_hdr;
@@ -187,7 +186,7 @@ void spice_caps_set(GArray *caps, guint32 cap, const gchar *desc);
 /* coroutine context */
 #define emit_main_context(object, event, args...)                                      \
     G_STMT_START {                                                                     \
-        if (IN_MAIN_CONTEXT) {                                                         \
+        if (coroutine_self_is_main()) {                                 \
             do_emit_main_context(G_OBJECT(object), event, &((struct event) { args })); \
         } else {                                                                       \
             g_signal_emit_main_context(G_OBJECT(object), do_emit_main_context,         \
