@@ -428,7 +428,7 @@ static void __write_io_word(Encoder *encoder)
 
 static void (*__write_io_word_ptr)(Encoder *encoder) = __write_io_word;
 
-static INLINE void write_io_word(Encoder *encoder)
+static inline void write_io_word(Encoder *encoder)
 {
     if (encoder->io_now == encoder->io_end) {
         __write_io_word_ptr(encoder); //disable inline optimizations
@@ -437,7 +437,7 @@ static INLINE void write_io_word(Encoder *encoder)
     *(encoder->io_now++) = encoder->io_word;
 }
 
-static INLINE void encode(Encoder *encoder, unsigned int word, unsigned int len)
+static inline void encode(Encoder *encoder, unsigned int word, unsigned int len)
 {
     int delta;
 
@@ -458,13 +458,13 @@ static INLINE void encode(Encoder *encoder, unsigned int word, unsigned int len)
     spice_assert((encoder->io_word & bppmask[encoder->io_available_bits]) == 0);
 }
 
-static INLINE void encode_32(Encoder *encoder, unsigned int word)
+static inline void encode_32(Encoder *encoder, unsigned int word)
 {
     encode(encoder, word >> 16, 16);
     encode(encoder, word & 0x0000ffff, 16);
 }
 
-static INLINE void flush(Encoder *encoder)
+static inline void flush(Encoder *encoder)
 {
     if (encoder->io_available_bits > 0 && encoder->io_available_bits != 32) {
         encode(encoder, 0, encoder->io_available_bits);
@@ -482,7 +482,7 @@ static void __read_io_word(Encoder *encoder)
 static void (*__read_io_word_ptr)(Encoder *encoder) = __read_io_word;
 
 
-static INLINE void read_io_word(Encoder *encoder)
+static inline void read_io_word(Encoder *encoder)
 {
     if (encoder->io_now == encoder->io_end) {
         __read_io_word_ptr(encoder); //disable inline optimizations
@@ -492,7 +492,7 @@ static INLINE void read_io_word(Encoder *encoder)
     encoder->io_next_word = *(encoder->io_now++);
 }
 
-static INLINE void decode_eatbits(Encoder *encoder, int len)
+static inline void decode_eatbits(Encoder *encoder, int len)
 {
     int delta;
 
@@ -512,7 +512,7 @@ static INLINE void decode_eatbits(Encoder *encoder, int len)
     encoder->io_word |= (encoder->io_next_word >> encoder->io_available_bits);
 }
 
-static INLINE void decode_eat32bits(Encoder *encoder)
+static inline void decode_eat32bits(Encoder *encoder)
 {
     decode_eatbits(encoder, 16);
     decode_eatbits(encoder, 16);
@@ -522,7 +522,7 @@ static INLINE void decode_eat32bits(Encoder *encoder)
 
 #ifdef RLE_STAT
 
-static INLINE void encode_ones(Encoder *encoder, unsigned int n)
+static inline void encode_ones(Encoder *encoder, unsigned int n)
 {
     unsigned int count;
 
@@ -718,7 +718,7 @@ static int decode_channel_run(Encoder *encoder, Channel *channel)
 
 #else
 
-static INLINE void encode_run(Encoder *encoder, unsigned int len)
+static inline void encode_run(Encoder *encoder, unsigned int len)
 {
     int odd = len & 1U;
     int msb;
@@ -739,7 +739,7 @@ static INLINE void encode_run(Encoder *encoder, unsigned int len)
     }
 }
 
-static INLINE unsigned int decode_run(Encoder *encoder)
+static inline unsigned int decode_run(Encoder *encoder)
 {
     unsigned int len = 0;
     int count;
@@ -761,7 +761,7 @@ static INLINE unsigned int decode_run(Encoder *encoder)
 #endif
 #endif
 
-static INLINE void init_decode_io(Encoder *encoder)
+static inline void init_decode_io(Encoder *encoder)
 {
     encoder->io_next_word = encoder->io_word = *(encoder->io_now++);
     encoder->io_available_bits = 0;
